﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Arction.WinForms.Charting;
using Arction.WinForms.Charting.Axes;
using Arction.WinForms.Charting.Series3D;
using Arction.WinForms.Charting.SeriesXY; 
using Arction.WinForms.Charting.Views;
using Arction.WinForms.Charting.Views.View3D;
using Arction.WinForms.Charting.Views.ViewXY; 


namespace WindowsFormsApplication1
{

    /// <summary>
    /// Example how to make SurfaceMeshSeries with geometry of a pipe, and coloring it by values 
    /// Copyright 2016 Arction Ltd. 
    /// Usage only in LightningChart related code. 
    /// </summary>
    public partial class ValueBased3DContouring : Form
    {
        LightningChartUltimate _chart;
        SurfaceMeshSeries3D _mesh;
        

        const int Columns = 150;
        const int Rows = 150;


        public ValueBased3DContouring()
        {
            InitializeComponent();

            _chart = new LightningChartUltimate();
            _chart.BeginUpdate(); 

            _chart.Parent = panel1;
            _chart.Dock = DockStyle.Fill;
            _chart.ActiveView = ActiveView.View3D; 
            
            View3D v = _chart.View3D;
            v.Camera.MinimumViewDistance = 50;
            v.Dimensions.Y = 200;
            v.Camera.ViewDistance = 300;
            
            
            //Create Surface mesh 
            _mesh = new SurfaceMeshSeries3D(v, Axis3DBinding.Primary, Axis3DBinding.Primary, Axis3DBinding.Primary);

            //CONTOURS 
            _mesh.ContourLineType = ContourLineType3D.ColorLineByValue; 
            _mesh.ContourLineWidth = 1.5;

            //WIREFRAME            
            _mesh.WireframeType = SurfaceWireframeType.Wireframe;
            _mesh.WireframeLineStyle.Width = 3; 
            _mesh.WireframeLineStyle.Color = Color.FromArgb(20, Color.Black);
            _mesh.WireframeOffset.SetValues(0, 0, 0);
            
            //FILL 
            _mesh.ColorSaturation = 80;
            _mesh.Fill = SurfaceFillStyle.PalettedByValue;         
            
            //Discontinuity is caused by surface mesh...end meeting each other, and data is not fluently continous over that. 

            InitializePalette();

            double[,] x = null;
            double[,] y = null;
            double[,] z = null;
            double[,] values = null;

            CreateGeometryPipe(out x, out y, out z);

            CreateValuesWaterDrops(out values);

            UpdateMesh(x, y, z, values);

            v.SurfaceMeshSeries3D.Add(_mesh);
            
            
            v.YAxisPrimary3D.Units.Text = "°C";

            _chart.EndUpdate(); 
        }

        

        void UpdateMesh(double[,] x, double[,] y, double[,] z, double[,] values)
        {
            

            int iCols = x.GetLength(0); 
            int iRows = x.GetLength(1); 
            SurfacePoint[,] data = new SurfacePoint[iCols,iRows];

            for (int iCol = 0; iCol < iCols; iCol++)
            {
                for (int iRow = 0; iRow < iRows; iRow++)
                {
                    data[iCol, iRow].X = x[iCol, iRow];
                    data[iCol, iRow].Y = y[iCol, iRow];
                    data[iCol, iRow].Z = z[iCol, iRow];
                    data[iCol, iRow].Value = values[iCol, iRow];
                } 

            }

            _mesh.Data = data;
        }


        
        
        //Pipe
        void CreateGeometryPipe(out double[,] x, out double[,] y, out double[,] z)
        {
            x = new double[Columns, Rows];
            y = new double[Columns, Rows];
            z = new double[Columns, Rows];
            double dXMin = 0;
            double dXMax = 100; 
            double dYMin = 0;
            double dYMax = 100; 
            double dZMin = 0;
            double dZMax = 100;
            double dCenterX = (dXMin + dXMax)/2.0; 
            double dCenterZ = (dZMin + dZMax)/2.0; 
            double dAngle; 
            double dYStep = (dYMax - dYMin) / (double)(Rows - 1);
            double dAngleStep = 2.0 * Math.PI / (double)(Columns - 1); //round tube 
            double dXRadius = 40; 
            double dZRadius = 40; 

            for (int iCol = 0; iCol < Columns; iCol++)
            {
                dAngle = (double)iCol * dAngleStep; 
                for (int iRow = 0; iRow < Rows; iRow++)
                {

                    x[iCol, iRow] = dCenterX + dXRadius * Math.Sin(dAngle);
                    y[iCol, iRow] = dYMin + (double)iRow * dYStep;
                    z[iCol, iRow] = dCenterZ + dZRadius * Math.Cos(dAngle);
                } 
            } 
        }

   
        void CreateValuesWaterDrops(out double[,] values)
        {
            values= WaterDropSurfaceDataGenerator.Generate(
                Columns, //size as nodes, in both X and Y
                new double[] { 0.5, 0.5, 0.7, 0.05 }, //Drop X positions in scale 0...1
                new double[] { 0.2, 0.9, 0.7, 0.95 }, //Drop Z positions in scale 0...1
                new double[] { 30, 50, 15, 50 }, //Amplitudes, as Y axis values
                40, //Offset level (mid-Y) 
                15); //Volatility, wave generating density
         
        }




        private void InitializePalette()
        {
            _mesh.ContourPalette.Steps.Clear();
            _mesh.ContourPalette.Steps.Add(new PaletteStep(_mesh.ContourPalette, Color.Blue, 0));
            _mesh.ContourPalette.Steps.Add(new PaletteStep(_mesh.ContourPalette, Color.Green, 25));
            _mesh.ContourPalette.Steps.Add(new PaletteStep(_mesh.ContourPalette, Color.Yellow, 50));
            _mesh.ContourPalette.Steps.Add(new PaletteStep(_mesh.ContourPalette, Color.Red, 75));
            _mesh.ContourPalette.Steps.Add(new PaletteStep(_mesh.ContourPalette, Color.White, 100));

            _mesh.ContourPalette.Type = PaletteType.Gradient;
        }

        
    }
}
